﻿using System;
using System.Collections.Generic;

namespace FactoryGetOrCreate
{
    // Inspired by Logger.getLogger(name) from the Java API
    public class Store<TKey, TValue> where TValue : new()
    {
        private readonly IDictionary<TKey, TValue> _data = new Dictionary<TKey, TValue>();

        // Strategy
        public Func<TValue> Creator { get; set; }

        public Store()
        {
            Creator = () => new TValue();
        }

        // Factory method
        public TValue GetOrCreate(TKey key)
        {
            if (_data.ContainsKey(key))
            {
                return _data[key];
            }
            //TValue value = new TValue();
            TValue value = Creator();
            _data[key] = value;
            return value;
        }

        public override string ToString()
        {
            return String.Join(", ", _data);
        }
    }
}
